import os
import time
import threading
import queue
import magic
from mutagen import File
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor, as_completed


def is_audio(filename)-> bool:
    print(filename)
    m = magic.detect_from_filename(filename=filename)
    return  True if m.mime_type.split('/')[0] == 'audio' else True if 'audio' in m.name.lower() else False


def get_audio_duration(file_path):
    """Отримує тривалість аудіофайлу через його метадані."""
    try:
        audio = File(file_path)
        if audio is not None and audio.info is not None:
            return audio.info.length  # Тривалість у секундах
    except Exception as e:
        print(f"Помилка при обробці {file_path}: {e}")
    return 0

def search_audio_files(folder_path, supported_extensions, file_queue, progress_bar, pb_lock):
    """
    Рекурсивно шукає аудіофайли в папці та додає їх у чергу для обробки.
    Після додавання кожного файлу оновлює total прогрес-бару.
    """
    for root, _, files in os.walk(folder_path):
        for file in files:
            full_path = os.path.join(root, file)
            if file.lower().endswith(supported_extensions):
                file_queue.put(full_path)
                # Оновлюємо total прогрес-бару
                with pb_lock:
                    progress_bar.total += 1
    # Пошук завершено

def process_audio_files(file_queue, total_duration, lock, progress_bar):
    """
    Обробляє файли з черги. Використовує sentinel (None) для завершення.
    """
    while True:
        file_path = file_queue.get()  # Блокуючий виклик
        if file_path is None:
            file_queue.task_done()
            break  # Отримали sentinel – завершуємо роботу потоку
        duration = get_audio_duration(file_path)
        with lock:
            total_duration[0] += duration
        progress_bar.update(1)  # Оновлюємо прогрес-бар: оброблено 1 файл
        file_queue.task_done()

def get_total_audio_duration(folder_path, num_threads=8):
    """Запускає пошук і обробку файлів паралельно з оптимізацією за допомогою sentinel-значень."""
    supported_extensions = (".mp3", ".wav", ".flac", ".ogg", ".m4a", ".aac", ".aif", ".aifc")
    file_queue = queue.Queue()
    total_duration = [0]  # Список для спільного доступу
    lock = threading.Lock()       # Для синхронізації оновлення total_duration
    pb_lock = threading.Lock()      # Для синхронізації оновлення прогрес-бару

    start_time = time.time()
    # Ініціалізуємо прогрес-бар з total=0 (буде оновлюватися під час пошуку)
    progress_bar = tqdm(desc="Обробка файлів", unit="файл", total=0, dynamic_ncols=True)

    # Запуск потоку пошуку файлів
    search_thread = threading.Thread(
        target=search_audio_files,
        args=(folder_path, supported_extensions, file_queue, progress_bar, pb_lock)
    )
    search_thread.start()
    search_thread.join()  # Чекаємо завершення пошуку

    # Додаємо sentinel для кожного процесного потоку
    for _ in range(num_threads):
        file_queue.put(None)

    # Запуск обробки файлів за допомогою ThreadPoolExecutor
    with ThreadPoolExecutor(max_workers=num_threads) as executor:
        futures = [executor.submit(process_audio_files, file_queue, total_duration, lock, progress_bar)
                   for _ in range(num_threads)]
        for future in as_completed(futures):
            future.result()  # Очікуємо завершення всіх потоків

    file_queue.join()
    progress_bar.close()
    elapsed_time = time.time() - start_time

    return total_duration[0], elapsed_time

def format_duration(seconds):
    """Форматує тривалість у форматі ЧЧ:ММ:СС."""
    hours = int(seconds // 3600)
    minutes = int((seconds % 3600) // 60)
    seconds = int(seconds % 60)
    return f"{hours:02}:{minutes:02}:{seconds:02}"

if __name__ == "__main__":
    folder = input("Введіть шлях до папки з аудіофайлами: ")
    total_seconds, elapsed_seconds = get_total_audio_duration(folder, num_threads=8)
    print("\n=== Результати ===")
    print(f"Загальна тривалість аудіофайлів: {format_duration(total_seconds)}")
    print(f"Час виконання скрипта: {format_duration(elapsed_seconds)}")

